/*
 * Channel.java
 * Part of the nodeNet problem set.
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see http://www.ai.mit.edu/projects/cs101, the
 * CS101 homepage or email las@ai.mit.edu.
 *
 * Copyright (C) 1998 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package nodeNet;

import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import javax.swing.*;
import cs101.util.*;
import java.io.*;

/** 
 * A Channel object is a NodeNetElement that connects two Nodes.  A
 * channel implements both the <code>InputChannel</code> and
 * <code>OutputChannel</code> interfaces.  Channels have an assocated
 * capacity and latency.  The capacity of a channel indicates how many
 * packets may be present before a <code>ChannelFullException</code>
 * is thrown.  The latency of a channel is a measure of how long (in
 * ms) a packet takes to travel from the input to the output.<p>
 *
 * Copyright (c) 1998 Massachusetts Institute of Technology
 *
 * @author Todd C. Parnell, tparnell@ai.mit.edu
 * @version $Id: Channel.java,v 1.2 1999/07/30 21:36:39 jsmthng Exp $ 
 */
public class Channel implements NodeNetElement, InputChannel, OutputChannel, Serializable {
  
  /** the maximum allowable latency */
  public final static int MAX_LATENCY = 5000;
  /** the maximum allowable capacity */
  public final static int MAX_CAPACITY = 50;
  
  private static ChannelConfigFrame cf;
  private static int ID;
  
  // JDK 1.2.2 appears to have a bug, wherein the .jar contains a
  // java.awt.Queue (although it doesn't exist in the documentation).
  // Explicitly labelling it cs101.util.Queue will pacify for now.  
  transient private final int myID = Channel.ID++;
  transient private cs101.util.Queue channel_objects = new DefaultQueue(), 
    timeStamps = new DefaultQueue();
  private int numberQueueObjects = 0; // for object serialization
  transient private float mvt_constant = 0;
  private int capacity = 10, latency = 0;
  private String name;    
  private Node startNode, stopNode;
  transient boolean selected, running;
  private boolean enabled = true, full, destroyed;
  transient private Object lock = new Object();
    
  // for painting performance
  private Point p1, p2;
  private Polygon poly1, poly2;
  private int x_diff, y_diff;
    
  //
  // Constructors
  //

  /**
   * Creates a new Channel with the given start and end nodes.  If
   * start and end are the same (as determined by
   * <code>equals</code>), throws SameNodeException.   
   */
  public Channel(Node start, Node end) throws SameNodeException {
    if (start.equals(end)) throw(new SameNodeException());

    this.name = "**UNNAMED**";
    this.startNode = start; 
    this.stopNode = end;
    this.channel_objects = new DefaultQueue();
    this.timeStamps = new DefaultQueue();
    this.startNode.addOutputChannel((OutputChannel)this);
    this.stopNode.addInputChannel((InputChannel)this);
	
    this.p1 = startNode.getPos();
    this.p2  = stopNode.getPos();
    if (this.p1 == null || this.p2 == null) return;
    int mid_x = (this.p1.x + this.p2.x) / 2;
    int mid_y = (this.p1.y + this.p2.y) / 2;
    this.x_diff = this.p2.x - this.p1.x;
    this.y_diff = this.p2.y - this.p1.y;
    float dist = (int)Math.sqrt(this.x_diff * this.x_diff + 
				this.y_diff * this.y_diff);
    int x_ratio = (int)Math.round((this.y_diff * 3) / dist);
    int y_ratio = (int)-Math.round((this.x_diff * 3) / dist);

    this.poly1 = new Polygon();
    this.poly1.addPoint(this.p1.x - x_ratio, this.p1.y - y_ratio);
    this.poly1.addPoint(this.p1.x + x_ratio, this.p1.y + y_ratio);
    this.poly1.addPoint(this.p2.x + x_ratio, this.p2.y + y_ratio);
    this.poly1.addPoint(this.p2.x - x_ratio, this.p2.y - y_ratio);
	
    this.poly2 = new Polygon();
    this.poly2.addPoint(mid_x - 3 * x_ratio, mid_y - 3 * y_ratio);
    this.poly2.addPoint(mid_x + 3 * x_ratio, mid_y + 3 * y_ratio);
    this.poly2.addPoint(p1.x + x_diff * 7 / 10, p1.y + y_diff * 7 / 10);
  }
    
  //
  // Public instance methods
  //
    
  /**
   * Insert an object into the channel.
   *
   * @param    o   The object to be inserted.  
   * @throws   ChannelFullException   if the number of objects in the
   * channel already equals or exceeds the capacity.
   * @throws   ChannelDisabledException  if the channel is currently
   * disabled
   */
  public void writeObject(Object o) throws ChannelFullException, ChannelDisabledException {
    synchronized (this.lock) {
      if (this.full)     throw(new ChannelFullException());
      if (!this.enabled) throw(new ChannelDisabledException());

      this.channel_objects.enqueue(o);
      this.timeStamps.enqueue(new Long(System.currentTimeMillis()));
      this.mvt_constant = 0;
      this.numberQueueObjects++;
      if (this.numberQueueObjects >= this.capacity) this.full = true;
    }
  }

  /**
   * Read an object from the channel.
   *
   * @return the oldest (in terms of insertion time) object in the channel
   *
   * @throws ChannelEmptyException if no objects are present in the
   * channel or the oldest object has not propigated through the
   * channel yet (due to a non-zero latency).
   *
   * @throws ChannelDisabledException if the channel is diabled  
   */
  public Object readObject() throws ChannelEmptyException, ChannelDisabledException {
    synchronized (this.lock) {
      if (!this.enabled) throw new ChannelDisabledException();
      if (this.channel_objects.size() == 0) throw new ChannelEmptyException();
      // temporarily get the timestamp for current packet
      long time = ((Long)this.timeStamps.peek()).longValue();   
      // check if packet's been around long enough
      if (System.currentTimeMillis() - time <= this.latency) {
	throw new ChannelEmptyException();
      }
      this.mvt_constant = 0.8f;
      // remove the timestamp
      this.timeStamps.dequeue();
      this.numberQueueObjects--;
      if (this.numberQueueObjects < this.capacity - 1) this.full = false;
      return(this.channel_objects.dequeue());
    }
  }
    
  public int getCapacity() { return this.capacity; }
  public void setCapacity(int cap) {
    if (cap > MAX_CAPACITY) cap = 50;
    if (cap < 0)  cap = 0;
    this.capacity = cap;
  }
    
  public int getLatency() { return this.latency; }
  public void setLatency(int lat) {
    if (lat > MAX_LATENCY) lat = 50;
    if (lat < 0) lat = 0;
    this.latency = lat;
  }

  public void setSelected(boolean b) { this.selected = b; }
  public boolean isSelected() { return(this.selected); }
    
  public void setEnabled(boolean b) { this.enabled = b; }
  public boolean isEnabled() { return(this.enabled); }

  public boolean isInside(int x, int y) { return(this.poly1.contains(x, y)); }
  public boolean isInside(Point p) { return(this.isInside(p.x, p.y)); }
    
  public void setName(String n) { this.name = n; }
  public String getName() { return this.name; }

  /**
   * Destroy the Channel.  <code>destroy()</code> should be called
   * whenever a Channel will no longer be used.  When destroyed,
   * Channels notify all connected Nodes of the impending destruction.
   * <I>Note: <code>Channel.finalize()</code> automatically calls
   * <code>destroy()</code></I>.
   */
  public void destroy() {
    this.stop();
    if (this.destroyed) return;
    this.destroyed = true;
    this.startNode.notifyOfDestruction(this);
    this.stopNode.notifyOfDestruction(this);
  }

  public boolean isDestroyed() { return this.destroyed; }

  /**
   * Notify this that a NodeNetElement has been destroyed.  Typically,
   * this method will be called by one of the two connected Nodes.  If
   * that is the case, the channel destroys itself.
   */
  public void notifyOfDestruction(NodeNetElement nne) {
    if (nne.equals(this.startNode) || nne.equals(this.stopNode)) {
      this.destroy();
    }
  }
    
  public boolean isEditable() { return true; }
  public void configure() {
    if (Channel.cf == null) Channel.cf = new ChannelConfigFrame();
    Channel.cf.setChannel(this); 
    Channel.cf.show();
  }
    
  public void paint(Graphics g) {
    if (this.destroyed) return;
    if (this.enabled && this.running) this.mvt_constant += .1f;
    if (this.mvt_constant >= 1) this.mvt_constant = 0;
    
    if (!this.enabled) g.setColor(Color.gray);
    else if (this.full) g.setColor(Color.red);
    else g.setColor(Color.black);
    g.fillPolygon(this.poly1);
    g.fillPolygon(this.poly2);
    if (this.selected) {
      g.setColor(Color.red);
      g.drawPolygon(this.poly2);
    }

    if (!this.enabled)  g.setColor(Color.black);
    else if (this.full) g.setColor(Color.pink);
    else /* std case*/  g.setColor(Color.red);
    int j = this.channel_objects.size();
    for (int i = 1; (i <= j) && ( i <= 6); i++) {
      int start_x = this.p1.x + 
	(int)(this.x_diff * (i + 2 + this.mvt_constant) / 12);
      int start_y = this.p1.y + 
	(int)(this.y_diff * (i + 2 + this.mvt_constant) / 12);
      g.fillOval(start_x, start_y, 8, 8);
    }
  }
    
  public void start() { this.running = true; }
  public void stop()  { this.running = false; }
  public void run() {}
    
  /** Determines if two Channels are equal. */
  public boolean equals(Object o) {
    if (!(o instanceof Channel)) return false;
    Channel c = (Channel)o;
    return (c.myID == this.myID);
  }

  //
  // Package instance methods
  //

  Node getStartNode() { return this.startNode; }
  Node getEndNode() { return this.stopNode; }
    

  //
  // Protected instance methods
  //

  protected void finalize() throws IOException {
    this.destroy();
  }

  //
  // Private instance methods
  //

  /**
   * Custom de-serialization.  Get NodeBehavior & color, then start
   * Thread.
   */
  private void readObject(ObjectInputStream in) 
       throws IOException, ClassNotFoundException {
    in.defaultReadObject();
    this.channel_objects = new DefaultQueue();
    this.timeStamps = new DefaultQueue();
    for (int i=0; i<this.numberQueueObjects; i++) {
      Object o = new Object();
      this.channel_objects.enqueue(o);
      this.timeStamps.enqueue(new Long(System.currentTimeMillis()));
    }
    this.lock = new Object();
  }

  /**
   * A configuration frame for Channels.
   */
  private class ChannelConfigFrame extends JFrame implements Serializable {
	
    transient Channel myChannel;
    JTextField bandwidth, latency;
    JButton ok, cancel;
	
    public ChannelConfigFrame() {
	    
      setTitle("Channel Configuration");
      JPanel p = new JPanel();
      p.setLayout(new FlowLayout());
      p.add(new Label("Capacity [0-" + Channel.MAX_CAPACITY + "]:"));
      bandwidth = new JTextField(4);
      p.add(bandwidth);
      p.add(new Label("Latency [0-" + Channel.MAX_LATENCY + "]:"));
      latency = new JTextField(4);
      p.add(latency);
      this.getContentPane().add("Center", p);
	    
      JPanel q = new JPanel();
      q.setLayout(new FlowLayout());
      this.ok = new JButton("OK");
      this.ok.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  configureChannel();
	  ChannelConfigFrame.this.setVisible(false);
	}
      });
      this.cancel = new JButton("Cancel");
      this.cancel.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  ChannelConfigFrame.this.setVisible(false);
	}
      });
      q.add(this.ok);
      q.add(this.cancel);
      this.getContentPane().add("South", q);

      this.pack();
	    
      this.addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	  ChannelConfigFrame.this.setVisible(false);
	}
      });
    }
	
    void setChannel(Channel newChannel) { 
      this.myChannel = newChannel;
      setTextFields();
    }
	
    void setTextFields() {
      bandwidth.setText(Integer.toString(getBandwidth()));
      latency.setText(Integer.toString(getLatency()));
    }
	
    void configureChannel() {
      if (this.myChannel == null) return;
      try {
	this.myChannel.setCapacity(Integer.parseInt(this.bandwidth.getText().trim()));
	this.myChannel.setLatency(Integer.parseInt(this.latency.getText().trim()));
	if (this.myChannel.capacity <= this.myChannel.numberQueueObjects)
	  this.myChannel.full = true;
	else 
	  this.myChannel.full = false;
      }
      catch(NumberFormatException e) {}
    }
	
    int getBandwidth() {
      if (this.myChannel == null) return 0;
      return this.myChannel.getCapacity();
    }
	
    int getLatency() {
      if (this.myChannel == null) return 0;
      return this.myChannel.getLatency();
    }
  } // class ChannelConfigFrame
} // class Channel

/*
 * $Log: Channel.java,v $
 * Revision 1.2  1999/07/30 21:36:39  jsmthng
 * Updated names of files to reflect change from 'BinSort' to 'nodeNet';
 * added InputChannelVector and OutputChannelVector classes (moving from
 * Vectors to ChannelVectors, unless we move back).
 *
 * Revision 1.1  1999/07/30 01:09:18  jsmthng
 * Renaming BinSort package to nodeNet; moving directories and files as
 * necessary.
 *
 * Revision 1.6  1999/07/27 16:57:44  las
 * Updating BinSort.Channel.java to use the new spelling of
 * cs101.util.Queue.peek() :o)
 *
 * Revision 1.5  1999/01/25 19:31:51  tparnell
 * Fixed packages.  cs101.util.Queue is now an interface, replaced with DefaultQueue
 *
 * Revision 1.4  1999/01/21 20:57:05  tparnell
 * it worked
 *
 * Revision 1.3  1998/08/12 19:29:28  tparnell
 * Another pass after comments from las & natashao.  Added support to
 * dynamically add NodeBehaviors.  Add keyboard shortcuts to menus.  Added
 * workaround to jdk bug relating to lighweight components.  Misc other
 * bugfixes.
 *
 * Revision 1.3  1998/08/10 17:45:50  tparnell
 * Revision to use JDK1.2 and Swing.  Redesign of GUI.  Removed old kludge
 * for file I/O and replaced with object serialization.  Channel no longer
 * requires animacy.  Removed unnessary dependencies between classes.
 * Added ability to configure channel's latency and capacity.  Added
 * javadoc to all files.  General code cleanup.
 *
 */
