/*
 * ControlPanel.java
 * Part of the nodeNet problem set.
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see http://www.ai.mit.edu/projects/cs101, the
 * CS101 homepage or email las@ai.mit.edu.
 *
 * Copyright (C) 1998 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package nodeNet;

import com.sun.java.swing.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.*;
import java.util.Vector;

/**
 * A <code>ControlPanel</code> object is a graphical display of
 * available Nodes for use with a <code>SimulationPanel</code>.<p>
 *
 * Copyright (c) 1998 Massachusetts Institute of Technology
 *
 * @author Todd C. Parnell, tparnell@ai.mit.edu
 * @version $Id: ControlPanel.java,v 1.3 1999/08/04 09:08:50 jsmthng Exp $ 
 */
public class ControlPanel extends Box {

  /** Current number of radio buttons. */
  private int count;
  private ButtonGroup bg;
  private Vector buttons;
  private JRadioButton dummy, savedButton;
  private boolean hackFlag;

  /**
   * Constructs a new ControlPanel with three node types: a generator
   * node, a terminator node, and a default connecting node.
   */
  public ControlPanel() {
    super(BoxLayout.Y_AXIS);
    this.bg = new ButtonGroup();
    this.buttons = new Vector();
    this.dummy = new JRadioButton();
    this.bg.add(this.dummy);
    try {
      this.newNodeType("nodeNet.GeneratorNodeBehavior", "Generator Node");
      this.newNodeType("nodeNet.DefaultNodeBehavior", "Default Connecting Node");
      this.newNodeType("nodeNet.TerminatorNodeBehavior", "Terminator Node");
    } catch (ClassNotFoundException cnfe) {
      System.err.println(cnfe);
    } catch (Exception e) {
      System.err.println(e);
    }
  }
  
  /** Get a new Node of the currently selected type. */
  public Node getNewInstance() {
    if (this.hackFlag) {
      this.hackFlag = false;
      return null;
    }

    for (int i=0; i<buttons.size(); i++) {
      NodeNetRadioButton nnrb = (NodeNetRadioButton)this.buttons.elementAt(i);
      if (nnrb.isSelected()) {
	return new Node(nnrb.newInstance(), nnrb.getColor());
      }
    }
    // something should be selected
    throw new RuntimeException("No radiobutton selected");
  }

  /** 
   * Adds a new NodeBehavior to the list of available types.
   *
   * @param type a new NodeBehavior to add
   * @throws ClassNotFoundException if type cannot be found
   * @throws IllegalArgumentException if type is not a valid
   * NodeBehavior
   */
  public void newNodeType(String type)
       throws ClassNotFoundException, IllegalArgumentException {
    this.newNodeType(type, type);
  }
 
  /** 
   * Adds a new NodeBehavior to the list of available types.
   *
   * @param type a new NodeBehavior to add
   * @param label the text label to use in the GUI
   * @throws ClassNotFoundException if type cannot be found
   * @throws IllegalArgumentException if type is not a valid
   * NodeBehavior
   */
  public void newNodeType(String nodeBehavior, String label) 
       throws ClassNotFoundException, IllegalArgumentException {
    NodeNetRadioButton nnrb;
    nnrb = new NodeNetRadioButton(nodeBehavior, label);
    // see if this behavior already exists
    if (this.buttons.contains(nnrb)) return;
    nnrb.setMnemonic(new Integer(this.count+1).toString().charAt(0));
    this.bg.add(nnrb);
    this.buttons.add(nnrb);
    this.add(nnrb);
    this.count++;  
    if (this.count == 1) nnrb.doClick();
    // force the Swing redraw...ugly
    this.invalidate();
    this.validate();
  }

  /**
   * Unfortunately, events on lightweight components "fall through" to
   * components below them in the Z order graphically.  In this app,
   * the SimulationPanel can sometimes be under a menu, which leads to
   * undeseriable behavior when the user clicks on the menu.  This
   * method is a small hack to prevent the SimulationPanel from adding
   * extra nodes when the user selects a menu.  I've reported this as a 
   * bug to Sun.<p>
   */
  void menuHack() { hackFlag = true; }

} // class ControlPanel

/*
 * $Log: ControlPanel.java,v $
 * Revision 1.3  1999/08/04 09:08:50  jsmthng
 * Added javadoc comments to InputChannelVector and OutputChannelVector;
 * finished updating the rest of the nodeNet package to reflect new
 * changes in name and code.
 *
 * Modified index.html to reflect the new nodeNet code, as well as to
 * clarify some parts of the problem set.
 *
 * Revision 1.1  1999/07/30 01:09:19  jsmthng
 * Renaming BinSort package to nodeNet; moving directories and files as
 * necessary.
 *
 * Revision 1.4  1999/01/21 20:57:05  tparnell
 * it worked
 *
 * Revision 1.3  1998/08/12 19:29:31  tparnell
 * Another pass after comments from las & natashao.  Added support to
 * dynamically add NodeBehaviors.  Add keyboard shortcuts to menus.  Added
 * workaround to jdk bug relating to lighweight components.  Misc other
 * bugfixes.
 *
 * Revision 1.3  1998/08/10 17:45:50  tparnell
 * Revision to use JDK1.2 and Swing.  Redesign of GUI.  Removed old kludge
 * for file I/O and replaced with object serialization.  Channel no longer
 * requires animacy.  Removed unnessary dependencies between classes.
 * Added ability to configure channel's latency and capacity.  Added
 * javadoc to all files.  General code cleanup.
 *
 */





