/*
 * Node.java
 * Part of the nodeNet problem set.
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see http://www.ai.mit.edu/projects/cs101, the
 * CS101 homepage or email las@ai.mit.edu.
 *
 * Copyright (C) 1998 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package nodeNet;

import com.sun.java.swing.*;
import javax.swing.*;
import java.util.*;
import java.io.*;
import java.awt.*;

/**
 * This class implements NodeNetElement.  It is responsible for the
 * GUI and maintains its own thread.  It relies on a class that
 * implements NodeBehavior to do anything.  It repeatedly calls that
 * class's act method from within a <code>while(true) {}
 * loop.</code><p>
 *
 * Copyright (c) 1998 Massachusetts Institute of Technology
 *
 * @see NodeNetElement
 * @see NodeBehavior
 * @author Todd C. Parnell, tparnell@ai.mit.edu
 * @version $Id: Node.java,v 1.3 1999/08/04 09:08:53 jsmthng Exp $ 
 */
public class Node implements NodeNetElement, Serializable {

  /** Diameter of the circle used to paint nodes. */
  public final static int GUISIZE = 20;
  public final static Color HIGHLIGHT_COLOR = Color.black;

  private static Color[] COLORS = { 
    Color.black, Color.blue, Color.gray, Color.green, Color.red, 
    Color.yellow, Color.pink };

  /** 
   * mapping between NodeBehaviors and Colors.  needs to be static so
   * Nodes can be serialized.  
   */
  private static Hashtable nbToColorMap; 
  static { Node.nbToColorMap = new Hashtable(); }

  final static int SLEEP_INTERVAL = 100;
  final static int DISABLED_SLEEP_INTERVAL = 1000;
  private static int ID;

  transient private Thread spirit;
  transient private final int myID = Node.ID++;
  transient private Color myColor;    
  private Point pos;
  transient boolean selected;             // highlighted in GUI?
  private boolean enabled = true,         // calling act?
    destroyed,                            // see note below
    stopped = true,                       // signal to Thread in run()
    count;                      // myBehavior instanceof CountingNodeBehavior
  private String name;
  transient private NodeBehavior myBehavior;
  private String myBehaviorStr;
  transient private CountingNodeBehavior countBehavior; // GUI optimization
  private InputChannelVector inputChannels;
  private OutputChannelVector outputChannels;    

  transient private Object lock = new Object(); // see note below

  //
  // Static methods
  //

  /** 
   * Returns the current Color associated with the given NodeBehavior.
   * If a mapping does not currently exist, a new one is created.
   *
   * @param behavior the NodeBehavior whose Color will be returned
   */
  public static Color getColorFromBehavior(String behavior) {
    if (!Node.nbToColorMap.containsKey(behavior)) {
      Node.nbToColorMap.put(behavior, 
			    Node.COLORS[Node.nbToColorMap.size()]);
    }
    return (Color)Node.nbToColorMap.get(behavior);
  }

  /**
   * Gets an Enumeration of all currently known NodeBehaviors.
   */
  public static Enumeration getBehaviors() {
    return Node.nbToColorMap.keys();
  }

  //
  // Constructors
  //

  public Node(NodeBehavior nb, Color c) {
    this.name = "*UNNAMED NODE*";
    this.inputChannels = new InputChannelVector();
    this.outputChannels = new OutputChannelVector();
    this.myColor = c;
    this.myBehavior = nb;
    this.myBehaviorStr = this.myBehavior.getClass().getName();
    if (this.myBehavior instanceof CountingNodeBehavior) {
      this.count = true;
      this.countBehavior = (CountingNodeBehavior)this.myBehavior;
    }
    else this.count = false;
 }

  //
  // Public methods
  //

  public void setPos(Point p) { this.pos = p; }
  public void setPos(int a, int b) { this.setPos(new Point(a, b)); }
  public Point getPos() { return(this.pos); }

  public void setSelected(boolean newState) { this.selected = newState; }
  public boolean isSelected() { return(this.selected); }

  public void setEnabled(boolean b) { this.enabled = b; }
  public boolean isEnabled() { return this.enabled; }
    
  public void setName(String n) { this.name = n; }
  public String getName() { return this.name; }
    
  public boolean isInside(Point p) { return(this.isInside(p.x, p.y)); }
  public boolean isInside(int x, int y) {
    return ((x > (this.pos.x - GUISIZE)) && (x < (this.pos.x + GUISIZE)) &&
	    (y > (this.pos.y - GUISIZE)) && (y < (this.pos.y + GUISIZE)));
  }

  public boolean isEditable() { return false; }
  /** Nodes cannot currently be configured. */
  public void configure() {
    Object[] options = { "OK" };
    JOptionPane.showOptionDialog(null, "This element cannot be configured.", 
				 "Network Simulation",
				 JOptionPane.DEFAULT_OPTION,
				 JOptionPane.WARNING_MESSAGE,
				 null, options, options[0]);
  }    

  /*
   * Note: the following 6 methods are synchronized on lock to enforce
   * synchronization between the transmitPacket method and calls here.
   * Students' code should return from the transmitPacket method...if it
   * doesn't adding and removing channels will not be possible.
   *
   * This is a change from earlier versions of nodeNet where students
   * could have written code that never returned from the act call.  
   */

  /**
   * Adds a new InputChannel.  If the specified InputChannel already
   * exists (as determined by the <code>equals</code> method), it is
   * not added again.  This condition is silently ignored.
   *
   * @param c the InputChannel to connect  
   */
  public void addInputChannel(InputChannel c) {
    synchronized (this.lock) {
      if (!this.inputChannels.contains(c)) 
	this.inputChannels.addElement(c);
    }
  }

  /**
   * Removes an InputChannel.  If the specified InputChannel is not a
   * current InputChannel for this Node (as determined by the
   * <code>equals</code> method), this method does nothing.
   *
   * @param c the InputChannel to disconnect
   */
  public void removeInputChannel(InputChannel c) {
    synchronized (this.lock) {
      boolean b = this.inputChannels.removeElement(c);
    }
  }

  /**
   * Adds a new OutputChannel.  If the specified OutputChannel already
   * exists (as determined by the <code>equals</code> method), it is
   * not added again.  This condition is silently ignored.
   *
   * @param c the OutputChannel to connect  
   */
  public void addOutputChannel(OutputChannel c) {
    synchronized (this.lock) {
      this.outputChannels.addElement(c);
    }
  }

  /**
   * Removes an OutputChannel.  If the specified OutputChannel is not a
   * current OutputChannel for this Node (as determined by the
   * <code>equals</code> method), this method does nothing.
   *
   * @param c the OutputChannel to disconnect
   */
  public void removeOutputChannel(OutputChannel c) {
    synchronized (this.lock) {
      boolean b = this.outputChannels.removeElement(c);
    }
  }
    
  /*
   * destroy() and notifyOfDestruction() use the destroyed field to
   * prevent cycles of destruction notifications to occur.  Use of the
   * field feels like a hack, but due to the distributed nature of
   * this app, is a good solution.
   */

  /**
   * Destroy the Node.  <code>destroy()</code> should be called
   * whenever a Node will no longer be used.  When destroyed, Nodes
   * notify all connected input and output channels of the impending
   * destruction and remove refrences to those channels.  Typically,
   * this will force their destruction as well.  <I>Note:
   * <code>Node.finalize()</code> automatically calls
   * <code>destroy()</code></I>.
   */
  public void destroy() {
    this.stop();
    this.destroyed = true; //eliminate concurrency problems

    //tell channels we're going away
    Enumeration enum = this.inputChannels.elements();
    while (enum.hasMoreElements()) {
      ((Channel)enum.nextElement()).notifyOfDestruction((NodeNetElement)this);
    }
    enum = this.outputChannels.elements();
    while (enum.hasMoreElements()) {
      ((Channel)enum.nextElement()).notifyOfDestruction((NodeNetElement)this);
    }
  }

  public boolean isDestroyed() { return this.destroyed; }

  /**
   * Notify this that a NodeNetElement has been destroyed.  Typically,
   * this method will be called by an input or output channel
   * connected to the node.  Once notified of destruction, all
   * references to the element are removed.
   */
  public void notifyOfDestruction(NodeNetElement bse) {
    // first see whether we've already been destroyed
    if (this.destroyed) return;
    if (bse instanceof InputChannel) {
      this.removeInputChannel( (InputChannel)bse );
    }
    if (bse instanceof OutputChannel) {
      this.removeOutputChannel( (OutputChannel)bse );
    }
  }
    
  public void paint(Graphics g) {
    if (this.destroyed || this.pos == null) return;
    // be polite & save current color
    Color tempColor = g.getColor();
    g.setColor(this.myColor);
    g.fillOval(this.pos.x - GUISIZE / 2, this.pos.y - GUISIZE / 2, 
	       GUISIZE, GUISIZE);
    if (this.selected) {
      g.setColor(HIGHLIGHT_COLOR);
      g.drawRect(this.pos.x - GUISIZE / 2, this.pos.y - GUISIZE / 2, 
		 GUISIZE, GUISIZE);
    }
    if (!this.enabled) {
      g.setColor(HIGHLIGHT_COLOR);
      g.drawLine(this.pos.x - GUISIZE / 2 - 5, this.pos.y - GUISIZE / 2 - 5,
		 this.pos.x + GUISIZE / 2 + 5, this.pos.y + GUISIZE / 2 + 5);
    }

    if (this.count) {
      String s = "" + this.countBehavior.getCount();
      g.setColor(Color.black);
      g.drawString(s, this.pos.x - GUISIZE, this.pos.y + GUISIZE);
    }
    // restore color
    g.setColor(tempColor);
  }
    
  /**
   * Starts the node running.  Once running, a node will repeatedly
   * call the <code>act</code> method for it's associated
   * NodeBehavior.
   */
  public void start() { 
    if (this.spirit == null) {
      this.stopped = false;
      this.spirit = new Thread(this);
      this.spirit.start();
    }
  }

  /**
   * Stops the node.
   */
  public void stop()  { 
    if (this.spirit == null) return;
    this.stopped = true;  // signal spirit
    // now wait until spirit has exited before returning
    while (this.spirit.isAlive()) {
      try { Thread.sleep(SLEEP_INTERVAL); }
      catch (InterruptedException ie) {
	// Do nothing...
      }
    }
    this.spirit = null;     // help the gc
  }

  /**
   * Repeatedly calls the <code>transmitPacket</code> method for the
   * associated NodeBehavior.  <I>Note: This method should never be
   * called directly.  Instead, use the <code>start()</code> method.</I>
   */
  public void run() {
    while(!this.stopped) {
      try { 
	if (this.enabled) Thread.sleep(SLEEP_INTERVAL); 
	else Thread.sleep(DISABLED_SLEEP_INTERVAL);
      } catch(InterruptedException e) {}

      if (this.enabled && this.myBehavior != null) { 
	synchronized (this.lock) {
	  this.myBehavior.transmitPacket(this.inputChannels, this.outputChannels); 
	}
      }
    }
  } // run()

  public boolean equals(Object o) {
    if (!(o instanceof Node)) return false;
    Node n = (Node)o;
    return (n.myID == this.myID);
  }

  //
  // Protected instance methods
  //

  protected void finalize() throws IOException {
    this.destroy();
  }
  
  //
  // Private methods
  //

  /**
   * Custom de-serialization.  Get NodeBehavior & color, then start
   * Thread.  
   */
  private void readObject(ObjectInputStream in) 
       throws IOException, ClassNotFoundException {
    in.defaultReadObject();
    this.lock = new Object();
    try {
      this.myBehavior = 
	(NodeBehavior)Class.forName(this.myBehaviorStr).newInstance();
      if (this.myBehavior instanceof CountingNodeBehavior) {
	this.count = true;
	this.countBehavior = (CountingNodeBehavior)this.myBehavior;
      }
      this.myColor = Node.getColorFromBehavior(this.myBehaviorStr);
    } catch (Exception e) {
      System.err.println("Error getting specified behavior. '" +
			 this.myBehaviorStr + "'.");
    }
  }

} // class Node

/*
 * $Log: Node.java,v $
 * Revision 1.3  1999/08/04 09:08:53  jsmthng
 * Added javadoc comments to InputChannelVector and OutputChannelVector;
 * finished updating the rest of the nodeNet package to reflect new
 * changes in name and code.
 *
 * Modified index.html to reflect the new nodeNet code, as well as to
 * clarify some parts of the problem set.
 *
 * Revision 1.1  1999/07/30 01:09:22  jsmthng
 * Renaming BinSort package to nodeNet; moving directories and files as
 * necessary.
 *
 * Revision 1.7  1999/01/21 20:57:06  tparnell
 * it worked
 *
 * Revision 1.6  1998/08/12 19:29:38  tparnell
 * Another pass after comments from las & natashao.  Added support to
 * dynamically add NodeBehaviors.  Add keyboard shortcuts to menus.  Added
 * workaround to jdk bug relating to lighweight components.  Misc other
 * bugfixes.
 *
 * Revision 1.3  1998/08/10 17:45:50  tparnell
 * Revision to use JDK1.2 and Swing.  Redesign of GUI.  Removed old kludge
 * for file I/O and replaced with object serialization.  Channel no longer
 * requires animacy.  Removed unnessary dependencies between classes.
 * Added ability to configure channel's latency and capacity.  Added
 * javadoc to all files.  General code cleanup.
 *
 */



