/*
 * NodeNetFrame.java
 * Part of the nodeNet problem set.
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see http://www.ai.mit.edu/projects/cs101, the
 * CS101 homepage or email las@ai.mit.edu.
 *
 * Copyright (C) 1998 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package nodeNet;

// import com.sun.java.swing.*;
// import com.sun.java.swing.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
 * The top level frame for nodeNet.  Creates a SimulationPanel and a
 * ControlPanel, sets up menus, and leaves the rest to the event
 * handlers.<p>
 *
 * Copyright (c) 1998 Massachusetts Institute of Technology
 *
 * @see SimulationPanel
 * @see ControlPanel
 *
 * @author Todd C. Parnell, tparnell@ai.mit.edu
 * @version $Id: NodeNetFrame.java,v 1.3 1999/08/04 09:08:54 jsmthng Exp $ 
 */
public class NodeNetFrame extends JFrame {

  private ControlPanel cp;
  private SimulationPanel sp;
  private JSplitPane jsp;

  /**
   * Constructs the GUI.
   *
   * @param nodeBehaviors a (possibly empty) list of NodeBehaviors to
   * add to the default list of NodeBehaviors.  Typically a student's
   * class.
   */
  public NodeNetFrame(String[] nodeBehaviors) {
    super("Network Simulation");

    this.cp = new ControlPanel();
    for (int i=0; i<nodeBehaviors.length; i++) {
      try {
	this.cp.newNodeType(nodeBehaviors[i]);
      } catch (ClassNotFoundException cnfe) {
	System.err.println("Unable to find the class '" +
			   nodeBehaviors[i] + "' ... exiting.");
	System.exit(1);
      } catch (IllegalArgumentException iae) {
	System.err.println("Error instantiating class '" +
			   nodeBehaviors[i] + "' ... exiting.");
	System.exit(1);
      }
    }
    this.sp = new SimulationPanel(this.cp);
    this.jsp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
			      this.cp,
			      this.sp);
    this.getContentPane().add(this.jsp);

    JMenuBar menubar = new JMenuBar();
    this.setJMenuBar(menubar);
    JMenu menu = new JMenu("File");
    menu.setMnemonic('f');
    menu.addMenuListener(new MenuListener() {
      public void menuSelected(MenuEvent e) {
	NodeNetFrame.this.cp.menuHack();
      }
      public void menuDeselected(MenuEvent e) {
      }
      public void menuCanceled(MenuEvent e) {
      }
    });

    menubar.add(menu);

    JMenuItem menuitem = new JMenuItem("New");
    menuitem.setMnemonic('n');
    menuitem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	String temp = "";
	try {
	  JFileChooser chooser = new JFileChooser();
	  int returnVal = chooser.showOpenDialog(NodeNetFrame.this);
	  if (returnVal != JFileChooser.APPROVE_OPTION ||
	      chooser.getSelectedFile() == null) return;
	  temp = chooser.getSelectedFile().getName();
	  int index = temp.indexOf(".");
	  if (index != -1) temp = temp.substring(0, index);
	  NodeNetFrame.this.cp.newNodeType(temp);
	} catch (java.security.AccessControlException ace) {
	  // we're running as an applet
	  NodeNetFrame.this.showAppletWarning();
	} catch (Exception e2) {
	  try {
	    NodeNetFrame.this.cp.newNodeType("nodeNet." + temp);
	  } catch (ClassNotFoundException cnfe) {
	    System.err.println("Unable to find the class '" +
			       temp + "' or 'nodeNet." + temp +"'.");
	  } catch (IllegalArgumentException iae) {
	    System.err.println("Error instantiating class '" +
			       temp + "' or nodeNet." + temp + "'.");
	  }
	}
      }
    });
    menu.add(menuitem);

    menuitem = new JMenuItem("Open...");
    menuitem.setMnemonic('o');
    menuitem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	SimulationPanel oldSP = sp;	
	try {
	  JFileChooser chooser = new JFileChooser();
	  int returnVal = chooser.showOpenDialog( NodeNetFrame.this );
	  if (returnVal != JFileChooser.APPROVE_OPTION ||
	      chooser.getSelectedFile() == null) return;
	  String file = chooser.getCurrentDirectory() +
	    System.getProperty("file.separator") +
	    chooser.getSelectedFile().getName();
	  FileInputStream fis = new FileInputStream(file);
	  ObjectInputStream in = new ObjectInputStream(fis);
	  NodeNetFrame.this.sp = (SimulationPanel)in.readObject();
	  NodeNetFrame.this.jsp.setRightComponent(NodeNetFrame.this.sp);
	  oldSP.reset();
	  sp.setControlPanel(cp);
	  in.close();
	} catch (java.security.AccessControlException ace) {
	  // we're running as an applet
	  NodeNetFrame.this.showAppletWarning();
	} catch (Exception e2) {
	  // print out the error
	  System.out.println(e2);
	}
      }
    });

    menu.add(menuitem);

    menu.addSeparator();

    menuitem = new JMenuItem("Save As...");
    menuitem.setMnemonic('s');
    menuitem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	try {
	  JFileChooser chooser = new JFileChooser();
	  int returnVal = chooser.showSaveDialog(NodeNetFrame.this);
	  if (returnVal != JFileChooser.APPROVE_OPTION || 
	      chooser.getSelectedFile() == null) return;
	  String file = chooser.getCurrentDirectory() +
	    System.getProperty("file.separator") +
	    chooser.getSelectedFile().getName();
	  FileOutputStream fos = new FileOutputStream(file);
	  ObjectOutputStream out = new ObjectOutputStream(fos);
	  out.writeObject(sp);
	  out.flush();
	  out.close();
	} catch (java.security.AccessControlException ace) {
	  // we're running as an applet
	  NodeNetFrame.this.showAppletWarning();
	} catch (Exception e2) {
	  System.out.println(e2);
	}
      }
    });
    menu.add(menuitem);

    menu.addSeparator();

    menuitem = new JMenuItem("Quit");
    menuitem.setMnemonic('q');
    menuitem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	try {
	  System.exit(0);
	} catch (java.security.AccessControlException ace) {
	  NodeNetFrame.this.showAppletWarning();
	}
      }
    });
    menu.add(menuitem);

    menu = new JMenu("Simulation");
    menu.setMnemonic('s');
    menubar.add(menu);
    menu.addMenuListener(new MenuListener() {
      public void menuSelected(MenuEvent e) {
	NodeNetFrame.this.cp.menuHack();
      }
      public void menuDeselected(MenuEvent e) {
      }
      public void menuCanceled(MenuEvent e) {
      }
    });

    menuitem = new JMenuItem("Start");
    menuitem.setMnemonic('s');
    menuitem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	sp.startSimulation();
      }
    });
    menu.add(menuitem);
    
    menuitem = new JMenuItem("Pause");
    menuitem.setMnemonic('p');
    menuitem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	sp.stopSimulation();
      }
    });
    menu.add(menuitem);
    
    menu.addSeparator();

    menuitem = new JMenuItem("Clear");
    menuitem.setMnemonic('c');
    menuitem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	sp.reset();
      }
    });
    menu.add(menuitem);


    menu = new JMenu("Selection");
    menu.setMnemonic('e');
    menubar.add(menu);
    menu.addMenuListener(new MenuListener() {
      public void menuSelected(MenuEvent e) {
	NodeNetFrame.this.cp.menuHack();
      }
      public void menuDeselected(MenuEvent e) {
      }
      public void menuCanceled(MenuEvent e) {
      }
    });
    
    menuitem = new JMenuItem("Enable");
    menuitem.setMnemonic('e');
    menuitem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	sp.setSelectionEnabled(true);
      }
    });
    menu.add(menuitem);

    menuitem = new JMenuItem("Disable");
    menuitem.setMnemonic('d');
    menuitem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	sp.setSelectionEnabled(false);
      }
    });
    menu.add(menuitem);

    menuitem = new JMenuItem("Configure");
    menuitem.setMnemonic('c');
    menuitem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	sp.configureSelection();
      }
    });
    menu.add(menuitem);

    menu.addSeparator();

    menuitem = new JMenuItem("Remove");
    menuitem.setMnemonic('r');
    menuitem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	sp.deleteSelection();
      }
    });
    menu.add(menuitem);

    this.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
	try {
	  System.exit(0);
	} catch (java.security.AccessControlException ace) {
	}
      }
    });
    
    this.pack();
    this.show();
  }

  /**
   * Pops up a warning dialog.
   */
  private void showAppletWarning() {
    Object[] options = { "Cancel" };
    JOptionPane.showOptionDialog(
      null, 			 
      "This action not available when running as an applet.", 
      "Network Simulation",
      JOptionPane.DEFAULT_OPTION,
      JOptionPane.WARNING_MESSAGE,
      null, options, options[0]);    
  }
}

/*
 * $Log: NodeNetFrame.java,v $
 * Revision 1.3  1999/08/04 09:08:54  jsmthng
 * Added javadoc comments to InputChannelVector and OutputChannelVector;
 * finished updating the rest of the nodeNet package to reflect new
 * changes in name and code.
 *
 * Modified index.html to reflect the new nodeNet code, as well as to
 * clarify some parts of the problem set.
 *
 * Revision 1.1  1999/07/30 01:09:23  jsmthng
 * Renaming BinSort package to nodeNet; moving directories and files as
 * necessary.
 *
 * Revision 1.5  1999/01/25 19:31:49  tparnell
 * Fixed packages.  cs101.util.Queue is now an interface, replaced with DefaultQueue
 *
 * Revision 1.4  1999/01/21 20:57:03  tparnell
 * it worked
 *
 * Revision 1.3  1998/08/14 16:17:43  tparnell
 * now appearing in applet version
 *
 * Revision 1.2  1998/08/12 19:29:26  tparnell
 * Another pass after comments from las & natashao.  Added support to
 * dynamically add NodeBehaviors.  Add keyboard shortcuts to menus.  Added
 * workaround to jdk bug relating to lighweight components.  Misc other
 * bugfixes.
 *
 * Revision 1.3  1998/08/10 17:45:50  tparnell
 * Revision to use JDK1.2 and Swing.  Redesign of GUI.  Removed old kludge
 * for file I/O and replaced with object serialization.  Channel no longer
 * requires animacy.  Removed unnessary dependencies between classes.
 * Added ability to configure channel's latency and capacity.  Added
 * javadoc to all files.  General code cleanup.
 *
 */