/*
 * NodeNetRadioButton.java
 * Part of the nodeNet problem set.
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see http://www.ai.mit.edu/projects/cs101, the
 * CS101 homepage or email las@ai.mit.edu.
 *
 * Copyright (C) 1998 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package nodeNet;

import com.sun.java.swing.*;
import javax.swing.*;
import java.lang.reflect.*;
import java.awt.*;

/**
 * The radiobuttons used by ControlPanel.  NodeNetRadioButtons are
 * associated with a NodeBehavior and can be asked to create a new
 * instance of that NodeBehavior.<p>
 *
 * Copyright (c) 1998 Massachusetts Institute of Technology
 *
 * @author Todd C. Parnell, tparnell@ai.mit.edu
 * @version $Id: NodeNetRadioButton.java,v 1.2 1999/07/30 21:36:44 jsmthng Exp $ 
 */
class NodeNetRadioButton extends JRadioButton {

  private Class behavior;
  private Color color;
  
  /**
   *  Attempt to create a new NodeNetRadioButton.  Throws exceptions
   *  when the nodeBehavior argument isn't a legal NodeBehavior.
   */
  public NodeNetRadioButton(String nodeBehavior, String label)
	 throws ClassNotFoundException, IllegalArgumentException {

    super(label);

    // grab the class indicated by arg
    this.behavior = Class.forName(nodeBehavior);
    // and check to see if it's legal
    Class nb = Class.forName("nodeNet.NodeBehavior");
    Class[] interfaces = this.behavior.getInterfaces();
    boolean found = false;
    for (int i=0; i<interfaces.length; i++) {
      if (interfaces[i].equals(nb)) {
	found = true;
	break;
      }
    }
    if (!found) {
      throw new IllegalArgumentException(nodeBehavior +
				     " does not implement NodeBehavior.");
    }
    int modifiers = this.behavior.getModifiers();
    if (Modifier.isAbstract(modifiers) || Modifier.isInterface(modifiers)) {
      throw new IllegalArgumentException(nodeBehavior +
				     " must be a non-abstract class.");
    }
    try {
      Object o = this.behavior.newInstance();
    } catch (IllegalAccessException iae) {
      throw new IllegalArgumentException(nodeBehavior +
				   " must have a zero-argument constructor.");
    } catch (InstantiationException ie) {
      // shouldn't happen (we know it's a non-abstract class)
      throw new RuntimeException("Error in ControlPanel.NodeNetIcon constructor");
    }    

    this.color = Node.getColorFromBehavior(nodeBehavior);
    this.setIcon(new NodeNetIcon(this.color, false));
    this.setSelectedIcon(new NodeNetIcon(this.color, true));
    this.setFocusPainted(false);
  } // constructor

  public NodeBehavior newInstance() {
    try {
      return (NodeBehavior)this.behavior.newInstance();
    } catch (InstantiationException ie) {
      // this should never happen, due to constructor checks
      throw new RuntimeException("InstantiationException in ControlPanel.NodeNetIcon.newInstance()");
    } catch (IllegalAccessException iae) {
      // should never happen, due to constructor checks
      throw new RuntimeException("IllegalAccessException in ControlPanel.NodeNetIcon.newInstance()");
      }      
  }

  public boolean equals(Object o) {
    if (!(o instanceof NodeNetRadioButton)) return false;
    NodeNetRadioButton nnrb = (NodeNetRadioButton)o;
    return nnrb.behavior.equals(this.behavior);
  }

  public Color getColor() { return this.color; }

  /**
   * A simple icon for use with the radiobuttons.  It remembers
   * whether it is selected or not and paints itself differently
   * depending.  It is semi-aware of how Nodes paint themselves.
   */
  private class NodeNetIcon implements Icon {
    
    private Color color;
    private boolean state;
  
    public NodeNetIcon (Color c, boolean s) {
      this.color = c;
      this.state = s;
    }

    public void paintIcon (Component c, Graphics g, int x, int y) {
      int width = getIconWidth();
      int height = getIconHeight();
      g.setColor(color);
      g.fillOval (x, y, width, height);
      if (this.state) {
	g.setColor(Color.black);
	g.drawRect(x, y, width, height);
      }
    }

    public int getIconWidth() {
      return Node.GUISIZE;
    }

    public int getIconHeight() { 
      return Node.GUISIZE;
    }
  } // class NodeNetIcon

} // class NodeNetRadioButton

/*
 * $Log: NodeNetRadioButton.java,v $
 * Revision 1.2  1999/07/30 21:36:44  jsmthng
 * Updated names of files to reflect change from 'BinSort' to 'nodeNet';
 * added InputChannelVector and OutputChannelVector classes (moving from
 * Vectors to ChannelVectors, unless we move back).
 *
 * Revision 1.1  1999/07/30 01:09:23  jsmthng
 * Renaming BinSort package to nodeNet; moving directories and files as
 * necessary.
 *
 * Revision 1.3  1999/01/21 20:57:03  tparnell
 * it worked
 *
 * Revision 1.2  1998/08/12 19:29:27  tparnell
 * Another pass after comments from las & natashao.  Added support to
 * dynamically add NodeBehaviors.  Add keyboard shortcuts to menus.  Added
 * workaround to jdk bug relating to lighweight components.  Misc other
 * bugfixes.
 *
 * Revision 1.3  1998/08/10 17:45:50  tparnell
 * Revision to use JDK1.2 and Swing.  Redesign of GUI.  Removed old kludge
 * for file I/O and replaced with object serialization.  Channel no longer
 * requires animacy.  Removed unnessary dependencies between classes.
 * Added ability to configure channel's latency and capacity.  Added
 * javadoc to all files.  General code cleanup.
 * 
 */