/*
 * SimulationPanel.java
 * Part of the nodeNet problem set.
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see http://www.ai.mit.edu/projects/cs101, the
 * CS101 homepage or email las@ai.mit.edu.
 *
 * Copyright (C) 1998 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package nodeNet;

import com.sun.java.swing.*;
import javax.swing.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;

/**
 * A graphical display of a group of NodeNetElements.  It manages
 * construction and destruction, as well as placement and
 * configuration of individual elements.<p>
 *
 * A SimulationPanel has an associated ControlPanel, which it queries
 * whenever a new element is to be added.
 *
 * Copyright (c) 1998 Massachusetts Institute of Technology
 *
 * @see NodeNetElement
 * @see ControlPanel
 *
 * @author Todd C. Parnell, tparnell@ai.mit.edu
 * @version $Id: SimulationPanel.java,v 1.3 1999/08/04 09:08:55 jsmthng Exp $ 
 */
class SimulationPanel extends JPanel implements Runnable, Serializable {

  /** dimensions */
  private static int WIDTH  = 400;
  private static int HEIGHT = 350;

  /** 
   * For serialization.  Change to a different number whenever
   * incompatible changes are made.
   */
  static final long serialVersionID = 1L;

  /** used for equals() */
  private static int ID_GENERATOR = 0;
  /** a unique identifier among all SimulationPanels in the VM */
  transient private final int ID = ID_GENERATOR++;

  /** our control panel.  refrenced only to get new nodes. */
  transient private ControlPanel myControl; 
  /** The elements we control */
  private Vector bsElements;
  /** flag to indicate whether the simulation (not the thread) is running */
  transient boolean running;
  /** element that is graphically selected */
  transient private NodeNetElement selectedNNE = null;
  /** the animacy */    
  transient private Thread spirit;
  /** flag to the Thread */
  private boolean stopped;
  /** how often to repaint */
  private final int repaint_pause = 100;
  /** when creating a Channel, the starting Node */
  transient private Node anchor;
    

  /**
   * Creates a new, empty SimulationPanel.  SimulationPanels start with
   * the simulation off.
   *
   * @param cp the ControlPanel to query for new Nodes.  
   */
  public SimulationPanel(ControlPanel cp) {
    super();

    this.myControl = cp;
    this.bsElements = new Vector();
    this.initListeners();
    this.spirit = new Thread(this);
    this.spirit.start();
  }

  private void initListeners() {
    this.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
	Point temp = e.getPoint();
	NodeNetElement nne = null;
	try { 
	  nne = whatsInside(temp);
	  SimulationPanel.this.setSelected(nne);
	  // if we get here, bse is non-null
	  if (SimulationPanel.this.anchor != null) {
	    if (nne instanceof Channel) {
	      // oops.  can't connect these up.
	      SimulationPanel.this.anchor = null; 
	      return;
	    }
	    // connect the dots with a new channel
	    Node n = (Node)nne;
	    if (channelExists(SimulationPanel.this.anchor, n)) return;
	    try {
	      Channel bsc = new Channel(SimulationPanel.this.anchor, n);
	      SimulationPanel.this.bsElements.addElement(bsc);
	      SimulationPanel.this.setSelected(bsc);
	      if (SimulationPanel.this.running) bsc.start();
	      return;
	    } catch (SameNodeException sne) { 
	    } finally {
	      SimulationPanel.this.anchor = null;
	    }
	  }
	  // if the user has double clicked on a node, set the anchor
	  if (e.getClickCount() > 1 && nne instanceof Node) {
	    SimulationPanel.this.anchor = (Node)nne;
	    return;
	  }	  
	}
	catch (IsInsideNothingException iine) {
	  // user clicked on empty space
	  if (SimulationPanel.this.anchor != null) {
	    // remove old anchor
	    SimulationPanel.this.anchor = null;
	    return;
	  } else if (SimulationPanel.this.myControl != null) {
	    // create a new node in this location
	    SimulationPanel.this.anchor = null;
	    Node n = SimulationPanel.this.myControl.getNewInstance();
	    if (n == null) return;  // see ControlPanel.menuHack()
 	    SimulationPanel.this.setSelected((NodeNetElement)n);
	    n.setPos(temp);
	    if (SimulationPanel.this.running) n.start();
	    SimulationPanel.this.bsElements.addElement(n);
	    return;
	  }
	}
	finally {
	  SimulationPanel.this.repaint();
	}
      }
    });
  } // initListeners()
	
  public Dimension getMinimumSize() { return(this.getPreferredSize()); }
  public Dimension getPreferredSize() {
    return new Dimension(SimulationPanel.WIDTH, SimulationPanel.HEIGHT);
  }
    
  public void paint(Graphics g) {

    Rectangle r = g.getClipBounds();
    g.clearRect(r.x, r.y, r.width, r.height);

    Enumeration enum;
    enum = this.bsElements.elements();
    // paint channels beneth nodes
    while (enum.hasMoreElements()) {
      NodeNetElement nne = (NodeNetElement)enum.nextElement();
      if (nne instanceof Channel) nne.paint(g);
    }
    enum = this.bsElements.elements();
    while (enum.hasMoreElements()) {
      NodeNetElement nne = (NodeNetElement)enum.nextElement();
      if (nne instanceof Node) nne.paint(g);
    }
  }

  /** 
   * Begins stepping through the simulation.  Starts all current
   * nodes and channels.
   */
  public void startSimulation() {
    if (this.running) return;
    Enumeration enum;
    enum = this.bsElements.elements();
    while (enum.hasMoreElements()) {
      ((NodeNetElement)enum.nextElement()).start();
    }
    this.running = true;
  }

  /** 
   * Stops the current simulation.<p>
   *
   * Note: this method waits for each Node and Channel to stop their
   * own execution before completing.  If the act method of any Node
   * does not return, this method will not return either.  
   */
  public void stopSimulation() {
    if (!this.running) return;
    Enumeration enum;
    enum = this.bsElements.elements();
    while (enum.hasMoreElements()) {
      ((NodeNetElement)enum.nextElement()).stop();
    }
    this.running = false;
  }

  /**
   * Stops the current simulation and removes all NodeNetElements.<p>
   *
   * Note: this method waits for each Node and Channel to stop their
   * own execution before completing.  If the act method of any Node
   * does not return, this method will not return either.  
   */
  public void reset() {
    this.stopSimulation();
    Enumeration enum;
    enum = this.bsElements.elements();
    while (enum.hasMoreElements()) {
      ((NodeNetElement)enum.nextElement()).destroy();
    }
    this.bsElements = new Vector();
  }

  /**
   * Enables or disables the currently selected NodeNetElement.
   * Disabled elements will still be visible in the simulation and
   * part of it.  However, disabled elements will not move packets.
   *
   * @param state the new state of the selection
   */
  public void setSelectionEnabled(boolean state) {
    if (this.selectedNNE == null) return;
    this.selectedNNE.setEnabled(state);
  }

  /**
   * Deletes the currently selected NodeNetElement.  If the selected
   * element is a Node, all channels connected to it are destroyed as
   * well.
   */
  public void deleteSelection() {
    if (this.selectedNNE == null) return;
    boolean wasRunning = this.running;
    if (wasRunning) this.stopSimulation();
    this.selectedNNE.destroy();
    Vector temp = new Vector();
    Enumeration enum;
    enum = this.bsElements.elements();
    while (enum.hasMoreElements()) {
      NodeNetElement nne = (NodeNetElement)enum.nextElement();
      if (!nne.isDestroyed()) temp.add(nne);
    }
    this.bsElements = temp;    
    if (wasRunning) this.startSimulation();
  }

  /**
   * Pops up a configuration dialog for the currently selected
   * element.
   */
  public void configureSelection() { 
    if (this.selectedNNE != null) this.selectedNNE.configure(); 
  }

  /**
   * Interactive control loop.  Just calls repaint()
   */	
  public void run() {
    try {
      while(!this.stopped) {
 	Thread.sleep(this.repaint_pause);
 	this.repaint();
      }
  }
     catch(InterruptedException e){}
  }
  
  public boolean equals(Object o) {
    if (!(o instanceof SimulationPanel)) return false;
    return (this.ID == ((SimulationPanel)o).ID);
  }

  //
  // Package instance methods
  //

  void setControlPanel(ControlPanel cp) {
    this.myControl = cp;
  }

  //
  // Protected instance methods
  //

  protected void finalize() throws IOException {
    this.stopped = true;
    this.stopSimulation();
  }
  
  //
  // Private instance methods
  //

  /** 
   * Switches the selected element. 
   *
   * @param bse the element to select
   */
  private void setSelected(NodeNetElement nne) {
    if (this.selectedNNE != null) this.selectedNNE.setSelected(false);
    this.selectedNNE = nne;
    if (this.selectedNNE != null) this.selectedNNE.setSelected(true);
  }
        
  /**
   * Determines the element at a particular Point on the GUI.
   *
   * @param p the point on the gui of interest
   */
  private NodeNetElement whatsInside(Point p) 
       throws IsInsideNothingException {
    NodeNetElement nne;
    Enumeration enum;
    enum = this.bsElements.elements();
    while (enum.hasMoreElements()) {
      nne = (NodeNetElement)enum.nextElement();
      if (nne.isInside(p)) return nne;
    }
    throw new IsInsideNothingException();
  }

  /**
   * Determines whether a channel exists between the specified nodes.
   */
  private boolean channelExists(Node start, Node finish) {
    Enumeration enum = this.bsElements.elements();
    Channel ch;
    while (enum.hasMoreElements()) {
      NodeNetElement nne = (NodeNetElement)enum.nextElement();
      if (!(nne instanceof Channel)) continue;
      ch = (Channel)nne;
      if (((ch.getStartNode()).equals(start)) && 
	  ((ch.getEndNode()).equals(finish))) {
	return true;
      }
    }
    return false;
  }

  private void readObject(ObjectInputStream in)
       throws IOException, ClassNotFoundException {
    // set up fields
    in.defaultReadObject();
    // listen to mouse events
    this.initListeners();
    // start the thread
    this.spirit = new Thread(this);
    this.spirit.start();
  }
}

/*
 * $Log: SimulationPanel.java,v $
 * Revision 1.3  1999/08/04 09:08:55  jsmthng
 * Added javadoc comments to InputChannelVector and OutputChannelVector;
 * finished updating the rest of the nodeNet package to reflect new
 * changes in name and code.
 *
 * Modified index.html to reflect the new nodeNet code, as well as to
 * clarify some parts of the problem set.
 *
 * Revision 1.1  1999/07/30 01:09:24  jsmthng
 * Renaming BinSort package to nodeNet; moving directories and files as
 * necessary.
 *
 * Revision 1.4  1999/01/21 20:57:07  tparnell
 * it worked
 *
 * Revision 1.3  1998/08/12 19:29:42  tparnell
 * Another pass after comments from las & natashao.  Added support to
 * dynamically add NodeBehaviors.  Add keyboard shortcuts to menus.  Added
 * workaround to jdk bug relating to lighweight components.  Misc other
 * bugfixes.
 *
 * Revision 1.3  1998/08/10 17:45:50  tparnell
 * Revision to use JDK1.2 and Swing.  Redesign of GUI.  Removed old kludge
 * for file I/O and replaced with object serialization.  Channel no longer
 * requires animacy.  Removed unnessary dependencies between classes.
 * Added ability to configure channel's latency and capacity.  Added
 * javadoc to all files.  General code cleanup.
 *
 */